@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2011 The HyperSpy developers
+# Copyright 2007-2021 The HyperSpy developers
 #
 # This file is part of  HyperSpy.
 #
@@ -16,17 +16,17 @@
 # You should have received a copy of the GNU General Public License
 # along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import division
+
 import copy
 
 import numpy as np
 from traits.api import Undefined
 
 from hyperspy.drawing.mpl_he import MPL_HyperExplorer
-from hyperspy.drawing import spectrum, utils
+from hyperspy.drawing import signal1d
 
 
-class MPL_HyperSpectrum_Explorer(MPL_HyperExplorer):
+class MPL_HyperSignal1D_Explorer(MPL_HyperExplorer):
 
     """Plots the current spectrum to the screen and a map with a cursor
     to explore the SI.
@@ -34,7 +34,7 @@ class MPL_HyperSpectrum_Explorer(MPL_HyperExplorer):
     """
 
     def __init__(self):
-        super(MPL_HyperSpectrum_Explorer, self).__init__()
+        super(MPL_HyperSignal1D_Explorer, self).__init__()
         self.xlabel = ''
         self.ylabel = ''
         self.right_pointer = None
@@ -54,9 +54,9 @@ class MPL_HyperSpectrum_Explorer(MPL_HyperExplorer):
             line.auto_update = value
         if self.pointer is not None:
             if value is True:
-                self.pointer.add_axes(self.navigator_plot.ax)
+                self.pointer.set_mpl_ax(self.navigator_plot.ax)
             else:
-                self.pointer.disconnect(self.navigator_plot.ax)
+                self.pointer.disconnect()
 
     @property
     def right_pointer_on(self):
@@ -73,73 +73,71 @@ class MPL_HyperSpectrum_Explorer(MPL_HyperExplorer):
         else:
             self.remove_right_pointer()
 
-    def plot_signal(self):
-        if self.signal_plot is not None:
-            self.signal_plot.plot()
-            return
+    def plot_signal(self, **kwargs):
+        super().plot_signal()
         # Create the figure
-        self.xlabel = '%s' % str(self.axes_manager.signal_axes[0])
-        if self.axes_manager.signal_axes[0].units is not Undefined:
-            self.xlabel += ' (%s)' % self.axes_manager.signal_axes[0].units
-        self.ylabel = 'Intensity'
-        self.axis = self.axes_manager.signal_axes[0].axis
-        sf = spectrum.SpectrumFigure(title=self.signal_title +
+        self.axis = self.axes_manager.signal_axes[0]
+        sf = signal1d.Signal1DFigure(title=self.signal_title +
                                      " Signal")
+        sf.axis = self.axis
+        if sf.ax is None:
+            sf.create_axis()
+        sf.axes_manager = self.axes_manager
+        self.xlabel = '{}'.format(self.axes_manager.signal_axes[0])
+        if self.axes_manager.signal_axes[0].units is not Undefined:
+            self.xlabel += ' ({})'.format(
+                self.axes_manager.signal_axes[0].units)
+        self.ylabel = self.quantity_label if self.quantity_label != '' \
+            else 'Intensity'
         sf.xlabel = self.xlabel
         sf.ylabel = self.ylabel
-        sf.axis = self.axis
-        sf.create_axis()
-        sf.axes_manager = self.axes_manager
+
         self.signal_plot = sf
         # Create a line to the left axis with the default indices
-        sl = spectrum.SpectrumLine()
-        sl.autoscale = True
+        sl = signal1d.Signal1DLine()
+        is_complex = np.iscomplexobj(self.signal_data_function())
         sl.data_function = self.signal_data_function
+        kwargs['data_function_kwargs'] = self.signal_data_function_kwargs
         sl.plot_indices = True
         if self.pointer is not None:
             color = self.pointer.color
         else:
             color = 'red'
         sl.set_line_properties(color=color, type='step')
-        # Add the line to the figure
-
+        # Add the line to the figure:
         sf.add_line(sl)
         # If the data is complex create a line in the left axis with the
         # default coordinates
-        sl = spectrum.SpectrumLine()
-        sl.data_function = self.signal_data_function
-        sl.plot_coordinates = True
-        sl.get_complex = any(np.iscomplex(sl.data_function()))
-        if sl.get_complex:
+        if is_complex:
+            sl = signal1d.Signal1DLine()
+            sl.data_function = self.signal_data_function
+            sl.plot_coordinates = True
+            sl._plot_imag = True
             sl.set_line_properties(color="blue", type='step')
             # Add extra line to the figure
             sf.add_line(sl)
 
         self.signal_plot = sf
-        sf.plot()
-        if self.navigator_plot is not None and sf.figure is not None:
-            utils.on_figure_window_close(self.navigator_plot.figure,
-                                         self._disconnect)
-            utils.on_figure_window_close(sf.figure,
-                                         self.close_navigator_plot)
-            self._key_nav_cid = \
+        sf.plot(**kwargs)
+        if sf.figure is not None:
+            if self.axes_manager.navigation_axes:
                 self.signal_plot.figure.canvas.mpl_connect(
-                    'key_press_event',
-                    self.axes_manager.key_navigator)
-            self._key_nav_cid = \
+                    'key_press_event', self.axes_manager.key_navigator)
+            if self.navigator_plot is not None:
+                sf.events.closed.connect(self.close_navigator_plot, [])
+                self.signal_plot.figure.canvas.mpl_connect(
+                    'key_press_event', self.key2switch_right_pointer)
                 self.navigator_plot.figure.canvas.mpl_connect(
-                    'key_press_event',
-                    self.axes_manager.key_navigator)
-            self.signal_plot.figure.canvas.mpl_connect(
-                'key_press_event', self.key2switch_right_pointer)
-            self.navigator_plot.figure.canvas.mpl_connect(
-                'key_press_event', self.key2switch_right_pointer)
+                    'key_press_event', self.key2switch_right_pointer)
+                self.navigator_plot.figure.canvas.mpl_connect(
+                    'key_press_event', self.axes_manager.key_navigator)
+            sf.events.closed.connect(self._on_signal_plot_closing, [])
 
     def key2switch_right_pointer(self, event):
         if event.key == "e":
             self.right_pointer_on = not self.right_pointer_on
 
-    def add_right_pointer(self):
+    def add_right_pointer(self, **kwargs):
         if self.signal_plot.right_axes_manager is None:
             self.signal_plot.right_axes_manager = \
                 copy.deepcopy(self.axes_manager)
@@ -147,20 +145,32 @@ class MPL_HyperSpectrum_Explorer(MPL_HyperExplorer):
             pointer = self.assign_pointer()
             self.right_pointer = pointer(
                 self.signal_plot.right_axes_manager)
-            self.right_pointer.size = self.pointer.size
+            # The following is necessary because e.g. a line pointer does not
+            # have size
+            if hasattr(self.pointer, "size"):
+                self.right_pointer.size = self.pointer.size
             self.right_pointer.color = 'blue'
-            self.right_pointer.add_axes(self.navigator_plot.ax)
-        rl = spectrum.SpectrumLine()
-        rl.autoscale = True
+            self.right_pointer.connect_navigate()
+            self.right_pointer.set_mpl_ax(self.navigator_plot.ax)
+
+        if self.right_pointer is not None:
+            for axis in self.axes_manager.navigation_axes[
+                    self._pointer_nav_dim:]:
+                self.signal_plot.right_axes_manager._axes[
+                    axis.index_in_array] = axis
+        rl = signal1d.Signal1DLine()
         rl.data_function = self.signal_data_function
         rl.set_line_properties(color=self.right_pointer.color,
                                type='step')
         self.signal_plot.create_right_axis()
-        self.signal_plot.add_line(rl, ax='right')
+        self.signal_plot.add_line(rl, ax='right', connect_navigation=True)
         rl.plot_indices = True
         rl.text_position = (1., 1.05,)
-        rl.plot()
+        rl.plot(**kwargs)
         self.right_pointer_on = True
+        # because we added the right axis, we need to redraw the canvas to
+        # update the background
+        self.signal_plot.figure.canvas.draw_idle()
 
     def remove_right_pointer(self):
         for line in self.signal_plot.right_ax_lines:
@@ -168,4 +178,3 @@ class MPL_HyperSpectrum_Explorer(MPL_HyperExplorer):
             line.close()
         self.right_pointer.close()
         self.right_pointer = None
-        self.navigator_plot.update()
