@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2011 The HyperSpy developers
+# Copyright 2007-2021 The HyperSpy developers
 #
 # This file is part of  HyperSpy.
 #
@@ -16,56 +16,73 @@
 # You should have received a copy of the GNU General Public License
 # along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
 
-import locale
-import time
-import datetime
+from datetime import datetime as dt
 import codecs
-import warnings
 import os
+import logging
 
 import numpy as np
+from traits.api import Undefined
 
-from hyperspy.misc.config_dir import os_name
 from hyperspy import Release
 from hyperspy.misc.utils import DictionaryTreeBrowser
 
+_logger = logging.getLogger(__name__)
+
 # Plugin characteristics
 # ----------------------
 format_name = 'MSA'
 description = ''
-full_suport = False
+full_support = False
 file_extensions = ('msa', 'ems', 'mas', 'emsa', 'EMS', 'MAS', 'EMSA', 'MSA')
 default_extension = 0
-
+# Writing capabilities
 writes = [(1, 0), ]
 # ----------------------
 
 # For a description of the EMSA/MSA format, incluiding the meaning of the
 # following keywords:
 # http://www.amc.anl.gov/ANLSoftwareLibrary/02-MMSLib/XEDS/EMMFF/EMMFF.IBM/Emmff.Total
+
+US_MONTHS_D2A = {
+    "01" : "JAN",
+    "02" : "FEB",
+    "03": "MAR",
+    "04": "APR",
+    "05": "MAY",
+    "06": "JUN",
+    "07": "JUL",
+    "08": "AUG",
+    "09": "SEP",
+    "10": "OCT",
+    "11": "NOV",
+    "12": "DEC", }
+
+US_MONTH_A2D = dict([reversed(i) for i in US_MONTHS_D2A.items()])
+
 keywords = {
     # Required parameters
-    'FORMAT': {'dtype': unicode, 'mapped_to': None},
-    'VERSION': {'dtype': unicode, 'mapped_to': None},
-    'TITLE': {'dtype': unicode, 'mapped_to': 'General.title'},
-    'DATE': {'dtype': unicode, 'mapped_to': None},
-    'TIME': {'dtype': unicode, 'mapped_to': None},
-    'OWNER': {'dtype': unicode, 'mapped_to': None},
+    'FORMAT': {'dtype': str, 'mapped_to': None},
+    'VERSION': {'dtype': str, 'mapped_to': None},
+    'TITLE': {'dtype': str, 'mapped_to': 'General.title'},
+    'DATE': {'dtype': str, 'mapped_to': None},
+    'TIME': {'dtype': str, 'mapped_to': None},
+    'OWNER': {'dtype': str, 'mapped_to': None},
     'NPOINTS': {'dtype': float, 'mapped_to': None},
     'NCOLUMNS': {'dtype': float, 'mapped_to': None},
-    'DATATYPE': {'dtype': unicode, 'mapped_to': None},
+    'DATATYPE': {'dtype': str, 'mapped_to': None},
     'XPERCHAN': {'dtype': float, 'mapped_to': None},
     'OFFSET': {'dtype': float, 'mapped_to': None},
     # Optional parameters
-    # Spectrum characteristics
-    'SIGNALTYPE': {'dtype': unicode, 'mapped_to':
+    # Signal1D characteristics
+    'SIGNALTYPE': {'dtype': str, 'mapped_to':
                    'Signal.signal_type'},
-    'XLABEL': {'dtype': unicode, 'mapped_to': None},
-    'YLABEL': {'dtype': unicode, 'mapped_to': None},
-    'XUNITS': {'dtype': unicode, 'mapped_to': None},
-    'YUNITS': {'dtype': unicode, 'mapped_to': None},
+    'XLABEL': {'dtype': str, 'mapped_to': None},
+    'YLABEL': {'dtype': str, 'mapped_to': None},
+    'XUNITS': {'dtype': str, 'mapped_to': None},
+    'YUNITS': {'dtype': str, 'mapped_to': None},
     'CHOFFSET': {'dtype': float, 'mapped_to': None},
-    'COMMENT': {'dtype': unicode, 'mapped_to': None},
+    'COMMENT': {'dtype': str, 'mapped_to': None},
     # Microscope
     'BEAMKV': {'dtype': float, 'mapped_to':
                'Acquisition_instrument.TEM.beam_energy'},
@@ -74,7 +91,7 @@ keywords = {
                  'Acquisition_instrument.TEM.beam_current'},
     'BEAMDIAM': {'dtype': float, 'mapped_to': None},
     'MAGCAM': {'dtype': float, 'mapped_to': None},
-    'OPERMODE': {'dtype': unicode, 'mapped_to': None},
+    'OPERMODE': {'dtype': str, 'mapped_to': None},
     'CONVANGLE': {'dtype': float, 'mapped_to':
                   'Acquisition_instrument.TEM.convergence_angle'},
 
@@ -82,20 +99,22 @@ keywords = {
     'THICKNESS': {'dtype': float, 'mapped_to':
                   'Sample.thickness'},
     'XTILTSTGE': {'dtype': float, 'mapped_to':
-                  'Acquisition_instrument.TEM.tilt_stage'},
+                  'Acquisition_instrument.TEM.Stage.tilt_alpha'},
     'YTILTSTGE': {'dtype': float, 'mapped_to': None},
     'XPOSITION': {'dtype': float, 'mapped_to': None},
     'YPOSITION': {'dtype': float, 'mapped_to': None},
     'ZPOSITION': {'dtype': float, 'mapped_to': None},
 
     # EELS
+    # in ms:
     'INTEGTIME': {'dtype': float, 'mapped_to':
-                  'Acquisition_instrument.TEM.Detector.EELS.exposure'},  # in ms
+                  'Acquisition_instrument.TEM.Detector.EELS.exposure'},
+    # in ms:
     'DWELLTIME': {'dtype': float, 'mapped_to':
-                  'Acquisition_instrument.TEM.Detector.EELS.dwell_time'},  # in ms
+                  'Acquisition_instrument.TEM.Detector.EELS.dwell_time'},
     'COLLANGLE': {'dtype': float, 'mapped_to':
                   'Acquisition_instrument.TEM.Detector.EELS.collection_angle'},
-    'ELSDET': {'dtype': unicode, 'mapped_to': None},
+    'ELSDET': {'dtype': str, 'mapped_to': None},
 
     # EDS
     'ELEVANGLE': {'dtype': float, 'mapped_to':
@@ -109,7 +128,8 @@ keywords = {
     'REALTIME': {'dtype': float, 'mapped_to':
                  'Acquisition_instrument.TEM.Detector.EDS.real_time'},
     'FWHMMNKA': {'dtype': float, 'mapped_to':
-                 'Acquisition_instrument.TEM.Detector.EDS.energy_resolution_MnKa'},
+                 'Acquisition_instrument.TEM.Detector.EDS.' +
+                 'energy_resolution_MnKa'},
     'TBEWIND': {'dtype': float, 'mapped_to': None},
     'TAUWIND': {'dtype': float, 'mapped_to': None},
     'TDEADLYR': {'dtype': float, 'mapped_to': None},
@@ -119,46 +139,61 @@ keywords = {
     'TBNWIND': {'dtype': float, 'mapped_to': None},
     'TDIWIND': {'dtype': float, 'mapped_to': None},
     'THCWIND': {'dtype': float, 'mapped_to': None},
-    'EDSDET': {'dtype': unicode, 'mapped_to':
+    'EDSDET': {'dtype': str, 'mapped_to':
                'Acquisition_instrument.TEM.Detector.EDS.EDS_det'},
 }
 
 
-def file_reader(filename, encoding='latin-1', **kwds):
+def parse_msa_string(string, filename=None):
+    """Parse an EMSA/MSA file content.
+
+    Parameters
+    ----------
+    string: string or file object
+        It must complain with the EMSA/MSA standard.
+    filename: string or None
+        The filename.
+
+    Returns:
+    --------
+    file_data_list: list
+        The list containts a dictionary that contains the parsed
+        information. It can be used to create a `:class:BaseSignal`
+        using `:func:hyperspy.io.dict2signal`.
+
+    """
+    if not hasattr(string, "readlines"):
+        string = string.splitlines()
     parameters = {}
     mapped = DictionaryTreeBrowser({})
-    with codecs.open(
-            filename,
-            encoding=encoding,
-            errors='replace') as spectrum_file:
-        y = []
-        # Read the keywords
-        data_section = False
-        for line in spectrum_file.readlines():
-            if data_section is False:
-                if line[0] == "#":
-                    try:
-                        key, value = line.split(': ')
-                        value = value.strip()
-                    except ValueError:
-                        key = line
-                        value = None
-                    key = key.strip('#').strip()
+    y = []
+    # Read the keywords
+    data_section = False
+    for line in string:
+        if data_section is False:
+            if line[0] == "#":
+                try:
+                    key, value = line.split(': ')
+                    value = value.strip()
+                except ValueError:
+                    key = line
+                    value = None
+                key = key.strip('#').strip()
 
-                    if key != 'SPECTRUM':
-                        parameters[key] = value
-                    else:
-                        data_section = True
-            else:
-                # Read the data
-                if line[0] != "#" and line.strip():
-                    if parameters['DATATYPE'] == 'XY':
-                        xy = line.replace(',', ' ').strip().split()
-                        y.append(float(xy[1]))
-                    elif parameters['DATATYPE'] == 'Y':
-                        data = [
-                            float(i) for i in line.replace(',', ' ').strip().split()]
-                        y.extend(data)
+                if key != 'SPECTRUM':
+                    parameters[key] = value
+                else:
+                    data_section = True
+        else:
+            # Read the data
+            if line[0] != "#" and line.strip():
+                if parameters['DATATYPE'] == 'XY':
+                    xy = line.replace(',', ' ').strip().split()
+                    y.append(float(xy[1]))
+                elif parameters['DATATYPE'] == 'Y':
+                    data = [
+                        float(i) for i in line.replace(',', ' ').strip().split()]
+                    y.extend(data)
     # We rewrite the format value to be sure that it complies with the
     # standard, because it will be used by the writer routine
     parameters['FORMAT'] = "EMSA/MAS Spectral Data File"
@@ -166,7 +201,7 @@ def file_reader(filename, encoding='latin-1', **kwds):
     # Convert the parameters to the right type and map some
     # TODO: the msa format seems to support specifying the units of some
     # parametes. We should add this feature here
-    for parameter, value in parameters.iteritems():
+    for parameter, value in parameters.items():
         # Some parameters names can contain the units information
         # e.g. #AZIMANGLE-dg: 90.
         if '-' in parameter:
@@ -177,15 +212,16 @@ def file_reader(filename, encoding='latin-1', **kwds):
         if clean_par in keywords:
             try:
                 parameters[parameter] = keywords[clean_par]['dtype'](value)
-            except:
+            except BaseException:
                 # Normally the offending mispelling is a space in the scientic
                 # notation, e.g. 2.0 E-06, so we try to correct for it
                 try:
                     parameters[parameter] = keywords[clean_par]['dtype'](
                         value.replace(' ', ''))
-                except:
-                    print("The %s keyword value, %s " % (parameter, value) +
-                          "could not be converted to the right type")
+                except BaseException:
+                    _logger.exception(
+                        "The %s keyword value, %s could not be converted to "
+                        "the right type", parameter, value)
 
             if keywords[clean_par]['mapped_to'] is not None:
                 mapped.set_item(keywords[clean_par]['mapped_to'],
@@ -193,55 +229,69 @@ def file_reader(filename, encoding='latin-1', **kwds):
                 if units is not None:
                     mapped.set_item(keywords[clean_par]['mapped_to'] +
                                     '_units', units)
-
-    # The data parameter needs some extra care
-    # It is necessary to change the locale to US english to read the date
-    # keyword
-    loc = locale.getlocale(locale.LC_TIME)
-    # Setting locale can raise an exception because
-    # their name depends on library versions, platform etc.
-    try:
-        if os_name == 'posix':
-            locale.setlocale(locale.LC_TIME, ('en_US', 'utf8'))
-        elif os_name == 'windows':
-            locale.setlocale(locale.LC_TIME, 'english')
+    if 'TIME' in parameters and parameters['TIME']:
         try:
-            H, M = time.strptime(parameters['TIME'], "%H:%M")[3:5]
-            mapped.set_item('General.time', datetime.time(H, M))
-        except:
-            if 'TIME' in parameters and parameters['TIME']:
-                print('The time information could not be retrieved')
+            time = dt.strptime(parameters['TIME'], "%H:%M")
+            mapped.set_item('General.time', time.time().isoformat())
+        except ValueError as e:
+            _logger.warning('Possible malformed TIME field in msa file. The time information could not be retrieved.: %s' % e)
+    else:
+        _logger.warning('TIME information missing.')
+
+    malformed_date_error = 'Possibly malformed DATE in msa file. The date information could not be retrieved.'
+    if "DATE" in parameters and parameters["DATE"]:
         try:
-            Y, M, D = time.strptime(parameters['DATE'], "%d-%b-%Y")[0:3]
-            mapped.set_item('General.date', datetime.date(Y, M, D))
-        except:
-            if 'DATE' in parameters and parameters['DATE']:
-                print('The date information could not be retrieved')
-    except:
-        warnings.warn("I couldn't write the date information due to"
-                      "an unexpected error. Please report this error to "
-                      "the developers")
-    locale.setlocale(locale.LC_TIME, loc)  # restore saved locale
+            day, month, year = parameters["DATE"].split("-")
+            if month.upper() in US_MONTH_A2D:
+                month = US_MONTH_A2D[month.upper()]
+                date = dt.strptime("-".join((day, month, year)), "%d-%m-%Y")
+                mapped.set_item('General.date', date.date().isoformat())
+            else:
+                    _logger.warning(malformed_date_error)
+        except ValueError as e: # Error raised if split does not return 3 elements in this case
+            _logger.warning(malformed_date_error + ": %s" % e)
 
-    axes = []
 
-    axes.append({
+    axes = [{
         'size': len(y),
         'index_in_array': 0,
         'name': parameters['XLABEL'] if 'XLABEL' in parameters else '',
         'scale': parameters['XPERCHAN'] if 'XPERCHAN' in parameters else 1,
         'offset': parameters['OFFSET'] if 'OFFSET' in parameters else 0,
         'units': parameters['XUNITS'] if 'XUNITS' in parameters else '',
-    })
-
-    mapped.set_item('General.original_filename', os.path.split(filename)[1])
+    }]
+    if filename is not None:
+        mapped.set_item('General.original_filename',
+                        os.path.split(filename)[1])
     mapped.set_item('Signal.record_by', 'spectrum')
     if mapped.has_item('Signal.signal_type'):
         if mapped.Signal.signal_type == 'ELS':
             mapped.Signal.signal_type = 'EELS'
+        if mapped.Signal.signal_type in ['EDX', 'XEDS']:
+            mapped.Signal.signal_type = 'EDS'
     else:
         # Defaulting to EELS looks reasonable
         mapped.set_item('Signal.signal_type', 'EELS')
+    if 'YUNITS' in parameters.keys():
+        yunits = "(%s)" % parameters['YUNITS']
+    else:
+        yunits = ""
+    if 'YLABEL' in parameters.keys():
+        quantity = "%s" % parameters['YLABEL']
+    else:
+        if mapped.Signal.signal_type == 'EELS':
+            quantity = 'Electrons'
+            if not yunits:
+                yunits = "(Counts)"
+        elif 'EDS' in mapped.Signal.signal_type:
+            quantity = 'X-rays'
+            if not yunits:
+                yunits = "(Counts)"
+        else:
+            quantity = ""
+    if quantity or yunits:
+        quantity_units = "%s %s" % (quantity, yunits)
+        mapped.set_item('Signal.quantity', quantity_units.strip())
 
     dictionary = {
         'data': np.array(y),
@@ -249,13 +299,24 @@ def file_reader(filename, encoding='latin-1', **kwds):
         'metadata': mapped.as_dictionary(),
         'original_metadata': parameters
     }
-    return [dictionary, ]
+    file_data_list = [dictionary, ]
+    return file_data_list
+
+
+def file_reader(filename, encoding='latin-1', **kwds):
+    with codecs.open(
+            filename,
+            encoding=encoding,
+            errors='replace') as spectrum_file:
+        return parse_msa_string(string=spectrum_file,
+                                filename=filename)
 
 
 def file_writer(filename, signal, format=None, separator=', ',
                 encoding='latin-1'):
     loc_kwds = {}
     FORMAT = "EMSA/MAS Spectral Data File"
+    md = signal.metadata
     if hasattr(signal.original_metadata, 'FORMAT') and \
             signal.original_metadata.FORMAT == FORMAT:
         loc_kwds = signal.original_metadata.as_dictionary()
@@ -267,27 +328,20 @@ def file_writer(filename, signal, format=None, separator=', ',
     else:
         if format is None:
             format = 'Y'
-        if signal.metadata.has_item("General.date"):
-            # Setting locale can raise an exception because
-            # their name depends on library versions, platform etc.
-            try:
-                loc = locale.getlocale(locale.LC_TIME)
-                if os_name == 'posix':
-                    locale.setlocale(locale.LC_TIME, ('en_US', 'latin-1'))
-                elif os_name == 'windows':
-                    locale.setlocale(locale.LC_TIME, 'english')
-                loc_kwds['DATE'] = signal.metadata.data.strftime(
-                    "%d-%b-%Y")
-                locale.setlocale(locale.LC_TIME, loc)  # restore saved locale
-            except:
-                warnings.warn("I couldn't write the date information due to"
-                              "an unexpected error. Please report this error to "
-                              "the developers")
+        if md.has_item("General.date"):
+            date = dt.strptime(md.General.date, "%Y-%m-%d")
+            date_str = date.strftime("%d-%m-%Y")
+            day, month, year = date_str.split("-")
+            month = US_MONTHS_D2A[month]
+            loc_kwds['DATE'] = "-".join((day, month, year)) 
+        if md.has_item("General.time"):
+            time = dt.strptime(md.General.time, "%H:%M:%S")
+            loc_kwds['TIME'] = time.strftime("%H:%M")
     keys_from_signal = {
         # Required parameters
         'FORMAT': FORMAT,
         'VERSION': '1.0',
-        #'TITLE' : signal.title[:64] if hasattr(signal, "title") else '',
+        # 'TITLE' : signal.title[:64] if hasattr(signal, "title") else '',
         'DATE': '',
         'TIME': '',
         'OWNER': '',
@@ -297,11 +351,16 @@ def file_writer(filename, signal, format=None, separator=', ',
         'SIGNALTYPE': signal.metadata.Signal.signal_type,
         'XPERCHAN': signal.axes_manager._axes[0].scale,
         'OFFSET': signal.axes_manager._axes[0].offset,
-        # Spectrum characteristics
+        # Signal1D characteristics
+
+        'XLABEL': signal.axes_manager._axes[0].name
+        if signal.axes_manager._axes[0].name is not Undefined
+        else "",
 
-        'XLABEL': signal.axes_manager._axes[0].name,
         #        'YLABEL' : '',
-        'XUNITS': signal.axes_manager._axes[0].units,
+        'XUNITS': signal.axes_manager._axes[0].units
+        if signal.axes_manager._axes[0].units is not Undefined
+        else "",
         #        'YUNITS' : '',
         'COMMENT': 'File created by HyperSpy version %s' % Release.version,
         # Microscope
@@ -328,11 +387,11 @@ def file_writer(filename, signal, format=None, separator=', ',
     }
 
     # Update the loc_kwds with the information retrieved from the signal class
-    for key, value in keys_from_signal.iteritems():
+    for key, value in keys_from_signal.items():
         if key not in loc_kwds or value != '':
             loc_kwds[key] = value
 
-    for key, dic in keywords.iteritems():
+    for key, dic in keywords.items():
 
         if dic['mapped_to'] is not None:
             if 'SEM' in signal.metadata.Signal.signal_type:
@@ -352,24 +411,24 @@ def file_writer(filename, signal, format=None, separator=', ',
             if key in loc_kwds:
                 del(loc_kwds[key])
 
-        f.write(u'#%-12s: %s\u000D\u000A' % ('FORMAT', loc_kwds.pop('FORMAT')))
+        f.write('#%-12s: %s\u000D\u000A' % ('FORMAT', loc_kwds.pop('FORMAT')))
         f.write(
-            u'#%-12s: %s\u000D\u000A' %
+            '#%-12s: %s\u000D\u000A' %
             ('VERSION', loc_kwds.pop('VERSION')))
         for keyword, value in loc_kwds.items():
-            f.write(u'#%-12s: %s\u000D\u000A' % (keyword, value))
+            f.write('#%-12s: %s\u000D\u000A' % (keyword, value))
 
-        f.write(u'#%-12s: Spectral Data Starts Here\u000D\u000A' % 'SPECTRUM')
+        f.write('#%-12s: Spectral Data Starts Here\u000D\u000A' % 'SPECTRUM')
 
         if format == 'XY':
             for x, y in zip(signal.axes_manager._axes[0].axis, signal.data):
                 f.write("%g%s%g" % (x, separator, y))
-                f.write(u'\u000D\u000A')
+                f.write('\u000D\u000A')
         elif format == 'Y':
             for y in signal.data:
                 f.write('%f%s' % (y, separator))
-                f.write(u'\u000D\u000A')
+                f.write('\u000D\u000A')
         else:
             raise ValueError('format must be one of: None, \'XY\' or \'Y\'')
 
-        f.write(u'#%-12s: End Of Data and File' % 'ENDOFDATA')
+        f.write('#%-12s: End Of Data and File' % 'ENDOFDATA')
