@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2011 The HyperSpy developers
+# Copyright 2007-2021 The HyperSpy developers
 #
 # This file is part of  HyperSpy.
 #
@@ -17,41 +17,33 @@
 # along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
 
 import copy
+import logging
 import warnings
 
-from hyperspy.model import Model
-from hyperspy.components import EELSCLEdge
-from hyperspy.components import PowerLaw
-from hyperspy.misc.ipython_tools import get_interactive_ns
-from hyperspy.defaults_parser import preferences
-import hyperspy.messages as messages
-from hyperspy import components
+from hyperspy import components1d
 from hyperspy._signals.eels import EELSSpectrum
+from hyperspy.components1d import EELSCLEdge, PowerLaw
+from hyperspy.docstrings.model import FIT_PARAMETERS_ARG
+from hyperspy.models.model1d import Model1D
+
+_logger = logging.getLogger(__name__)
 
 
-def _give_me_delta(master, slave):
-    return lambda x: x + slave - master
+class EELSModel(Model1D):
 
-
-def _give_me_idelta(master, slave):
-    return lambda x: x - slave + master
-
-
-class EELSModel(Model):
-
-    """Build a fit a model
+    """Build an EELS model
 
     Parameters
     ----------
-    spectrum : an Spectrum (or any Spectrum subclass) instance
-    auto_background : boolean
+    spectrum : a Signal1D (or any Signal1D subclass) instance
+    auto_background : bool
         If True, and if spectrum is an EELS instance adds automatically
         a powerlaw to the model and estimate the parameters by the
         two-area method.
-    auto_add_edges : boolean
+    auto_add_edges : bool
         If True, and if spectrum is an EELS instance, it will
         automatically add the ionization edges as defined in the
-        Spectrum instance. Adding a new element to the spectrum using
+        Signal1D instance. Adding a new element to the spectrum using
         the components.EELSSpectrum.add_elements method automatically
         add the corresponding ionisation edges to the model.
     ll : {None, EELSSpectrum}
@@ -63,65 +55,94 @@ class EELSModel(Model):
         The GOS to use when auto adding core-loss EELS edges.
         If None it will use the Hartree-Slater GOS if
         they are available, otherwise it will use the hydrogenic GOS.
+    dictionary : {dict, None}
+        A dictionary to be used to recreate a model. Usually generated using
+        :meth:`hyperspy.model.as_dictionary`
 
     """
 
-    def __init__(self, spectrum, auto_background=True,
+    def __init__(self, signal1D, auto_background=True,
                  auto_add_edges=True, ll=None,
-                 GOS=None, *args, **kwargs):
-        Model.__init__(self, spectrum, *args, **kwargs)
+                 GOS=None, dictionary=None):
+        Model1D.__init__(self, signal1D)
+
+        # When automatically setting the fine structure energy regions,
+        # the fine structure of an EELS edge component is automatically
+        # disable if the next ionisation edge onset distance to the
+        # higher energy side of the fine structure region is lower that
+        # the value of this parameter
+        self._min_distance_between_edges_for_fine_structure = 0
+        self._preedge_safe_window_width = 2
+        self.signal1D = signal1D
         self._suspend_auto_fine_structure_width = False
         self.convolved = False
         self.low_loss = ll
         self.GOS = GOS
-        self.edges = list()
+        self.edges = []
+        self._background_components = []
+        if dictionary is not None:
+            auto_background = False
+            auto_add_edges = False
+            self._load_dictionary(dictionary)
+
         if auto_background is True:
-            interactive_ns = get_interactive_ns()
             background = PowerLaw()
-            background.name = 'background'
-            interactive_ns['background'] = background
             self.append(background)
 
-        if self.spectrum.subshells and auto_add_edges is True:
+        if self.signal.subshells and auto_add_edges is True:
             self._add_edges_from_subshells_names()
 
     @property
-    def spectrum(self):
-        return self._spectrum
+    def signal1D(self):
+        return self._signal
 
-    @spectrum.setter
-    def spectrum(self, value):
+    @signal1D.setter
+    def signal1D(self, value):
         if isinstance(value, EELSSpectrum):
-            self._spectrum = value
-            self.spectrum._are_microscope_parameters_missing()
+            self._signal = value
         else:
             raise ValueError(
                 "This attribute can only contain an EELSSpectrum "
                 "but an object of type %s was provided" %
                 str(type(value)))
 
-    def _touch(self):
-        """Run model setup tasks
+    def append(self, component):
+        super(EELSModel, self).append(component)
+        if isinstance(component, EELSCLEdge):
+            tem = self.signal.metadata.Acquisition_instrument.TEM
+            component.set_microscope_parameters(
+                E0=tem.beam_energy,
+                alpha=tem.convergence_angle,
+                beta=tem.Detector.EELS.collection_angle,
+                energy_scale=self.axis.scale)
+            component.energy_scale = self.axis.scale
+            component._set_fine_structure_coeff()
+        self._classify_components()
 
-        This function must be called everytime that we add or remove components
-        from the model.
-        It creates the bookmarks self.edges and self._background_components and
-        configures the edges by setting the energy_scale attribute and setting
-        the fine structure.
+    append.__doc__ = Model1D.append.__doc__
+
+    def remove(self, component):
+        super(EELSModel, self).remove(component)
+        self._classify_components()
+
+    remove.__doc__ = Model1D.remove.__doc__
+
+    def _classify_components(self):
+        """Classify components between background and ionization edge
+        components.
+
+        This method should be called everytime that components are added and
+        removed. An ionization edge becomes background when its onset falls to
+        the left of the first non-masked energy channel. The ionization edges
+        are stored in a list in the `edges` attribute. They are sorted by
+        increasing `onset_energy`. The background components are stored in
+        `_background_components`.
 
         """
-        self._Model__touch()
         self.edges = []
         self._background_components = []
         for component in self:
             if isinstance(component, EELSCLEdge):
-                component.set_microscope_parameters(
-                    E0=self.spectrum.metadata.Acquisition_instrument.TEM.beam_energy,
-                    alpha=self.spectrum.metadata.Acquisition_instrument.TEM.convergence_angle,
-                    beta=self.spectrum.metadata.Acquisition_instrument.TEM.Detector.EELS.collection_angle,
-                    energy_scale=self.axis.scale)
-                component.energy_scale = self.axis.scale
-                component._set_fine_structure_coeff()
                 if component.onset_energy.value < \
                         self.axis.axis[self.channel_switches][0]:
                     component.isbackground = True
@@ -130,8 +151,6 @@ class EELSModel(Model):
                 else:
                     component.fine_structure_active = False
                     component.fine_structure_coeff.free = False
-                    component.backgroundtype = "edge"
-                    self._background_components.append(component)
             elif (isinstance(component, PowerLaw) or
                   component.isbackground is True):
                 self._background_components.append(component)
@@ -144,9 +163,10 @@ class EELSModel(Model):
         elif len(self._background_components) == 1:
             self._backgroundtype = \
                 self._background_components[0].__repr__()
-            if self._firstimetouch and self.edges:
+            bg = self._background_components[0]
+            if isinstance(bg, PowerLaw) and self.edges and not \
+               bg.A.map["is_set"].any():
                 self.two_area_background_estimation()
-                self._firstimetouch = False
 
     @property
     def _active_edges(self):
@@ -156,29 +176,27 @@ class EELSModel(Model):
     def _active_background_components(self):
         return [bc for bc in self._background_components if bc.active]
 
-    def _add_edges_from_subshells_names(self, e_shells=None,
-                                        copy2interactive_ns=True):
+    def _add_edges_from_subshells_names(self, e_shells=None):
         """Create the Edge instances and configure them appropiately
         Parameters
         ----------
         e_shells : list of strings
-        copy2interactive_ns : bool
-            If True, variables with the format Element_Shell will be
-            created in IPython's interactive shell
         """
-        interactive_ns = get_interactive_ns()
+        if self.signal._are_microscope_parameters_missing():
+            raise ValueError(
+                "The required microscope parameters are not defined in "
+                "the EELS spectrum signal metadata. Use "
+                "``set_microscope_parameters`` to set them."
+            )
         if e_shells is None:
-            e_shells = list(self.spectrum.subshells)
+            e_shells = list(self.signal.subshells)
         e_shells.sort()
         master_edge = EELSCLEdge(e_shells.pop(), self.GOS)
         # If self.GOS was None, the GOS is set by eels_cl_edge so
         # we reassing the value of self.GOS
         self.GOS = master_edge.GOS._name
         self.append(master_edge)
-        interactive_ns[self[-1].name] = self[-1]
         element = master_edge.element
-        interactive_ns[element] = []
-        interactive_ns[element].append(self[-1])
         while len(e_shells) > 0:
             next_element = e_shells[-1].split('_')[0]
             if next_element != element:
@@ -195,21 +213,16 @@ class EELSModel(Model):
                 edge = EELSCLEdge(e_shells.pop(), GOS=self.GOS)
 
                 edge.intensity.twin = master_edge.intensity
-                delta = _give_me_delta(master_edge.GOS.onset_energy,
-                                       edge.GOS.onset_energy)
-                idelta = _give_me_idelta(master_edge.GOS.onset_energy,
-                                         edge.GOS.onset_energy)
-                edge.onset_energy.twin_function = delta
-                edge.onset_energy.twin_inverse_function = idelta
                 edge.onset_energy.twin = master_edge.onset_energy
+                edge.onset_energy.twin_function_expr = "x + {}".format(
+                    (edge.GOS.onset_energy - master_edge.GOS.onset_energy))
                 edge.free_onset_energy = False
                 self.append(edge)
-                if copy2interactive_ns is True:
-                    interactive_ns[edge.name] = edge
-                    interactive_ns[element].append(edge)
 
     def resolve_fine_structure(
-            self, preedge_safe_window_width=preferences.EELS.preedge_safe_window_width, i1=0):
+            self,
+            preedge_safe_window_width=2,
+            i1=0):
         """Adjust the fine structure of all edges to avoid overlapping
 
         This function is called automatically everytime the position of an edge
@@ -219,7 +232,8 @@ class EELSModel(Model):
         ----------
         preedge_safe_window_width : float
             minimum distance between the fine structure of an ionization edge
-            and that of the following one.
+            and that of the following one. Default 2 (eV).
+
         """
 
         if self._suspend_auto_fine_structure_width is True:
@@ -237,129 +251,114 @@ class EELSModel(Model):
                     i2 < len(self._active_edges) - 1):
                 i2 += 1
             if self._active_edges[i2].fine_structure_active is True:
-                distance_between_edges = self._active_edges[i2].onset_energy.value - \
-                    self._active_edges[i1].onset_energy.value
-                if self._active_edges[i1].fine_structure_width > distance_between_edges - \
-                        preedge_safe_window_width:
+                distance_between_edges = (
+                    self._active_edges[i2].onset_energy.value -
+                    self._active_edges[i1].onset_energy.value)
+                if (self._active_edges[i1].fine_structure_width >
+                        distance_between_edges -
+                        self._preedge_safe_window_width):
+                    min_d = self._min_distance_between_edges_for_fine_structure
                     if (distance_between_edges -
-                            preedge_safe_window_width) <= \
-                            preferences.EELS.min_distance_between_edges_for_fine_structure:
-                        print " Automatically desactivating the fine \
-                        structure of edge number", i2 + 1, "to avoid conflicts\
-                         with edge number", i1 + 1
+                            self._preedge_safe_window_width) <= min_d:
+                        _logger.info((
+                            "Automatically deactivating the fine structure "
+                            "of edge number %d to avoid conflicts with edge "
+                            "number %d") % (i2 + 1, i1 + 1))
                         self._active_edges[i2].fine_structure_active = False
                         self._active_edges[
                             i2].fine_structure_coeff.free = False
                         self.resolve_fine_structure(i1=i2)
                     else:
-                        new_fine_structure_width = distance_between_edges - \
-                            preedge_safe_window_width
-                        print "Automatically changing the fine structure \
-                        width of edge", i1 + 1, "from", \
-                            self._active_edges[i1].fine_structure_width, "eV to", new_fine_structure_width, \
-                            new_fine_structure_width, \
-                            "eV to avoid conflicts with edge number", i2 + 1
-                        self._active_edges[
-                            i1].fine_structure_width = new_fine_structure_width
+                        new_fine_structure_width = (
+                            distance_between_edges -
+                            self._preedge_safe_window_width)
+                        _logger.info((
+                            "Automatically changing the fine structure "
+                            "width of edge %d from %s eV to %s eV to avoid "
+                            "conflicts with edge number %d") % (
+                            i1 + 1,
+                            self._active_edges[i1].fine_structure_width,
+                            new_fine_structure_width,
+                            i2 + 1))
+                        self._active_edges[i1].fine_structure_width = \
+                            new_fine_structure_width
                         self.resolve_fine_structure(i1=i2)
                 else:
                     self.resolve_fine_structure(i1=i2)
         else:
             return
 
-    def fit(self, fitter=None, method='ls', grad=False,
-            bounded=False, ext_bounding=False, update_plot=False,
-            kind='std', **kwargs):
-        """Fits the model to the experimental data
+    def fit(self, kind="std", **kwargs):
+        """Fits the model to the experimental data.
+
+        Read more in the :ref:`User Guide <model.fitting>`.
 
         Parameters
         ----------
-        fitter : {None, "leastsq", "odr", "mpfit", "fmin"}
-            The optimizer to perform the fitting. If None the fitter
-            defined in the Preferences is used. leastsq is the most
-            stable but it does not support bounding. mpfit supports
-            bounding. fmin is the only one that supports
-            maximum likelihood estimation, but it is less robust than
-            the Levenberg–Marquardt based leastsq and mpfit, and it is
-            better to use it after one of them to refine the estimation.
-        method : {'ls', 'ml'}
-            Choose 'ls' (default) for least squares and 'ml' for
-            maximum-likelihood estimation. The latter only works with
-            fitter = 'fmin'.
-        grad : bool
-            If True, the analytical gradient is used if defined to
-            speed up the estimation.
-        ext_bounding : bool
-            If True, enforce bounding by keeping the value of the
-            parameters constant out of the defined bounding area.
-        bounded : bool
-            If True performs bounded optimization if the fitter
-            supports it. Currently only mpfit support bounding.
-        update_plot : bool
-            If True, the plot is updated during the optimization
-            process. It slows down the optimization but it permits
-            to visualize the optimization evolution.
-        kind : {'std', 'smart'}
-            If 'std' (default) performs standard fit. If 'smart'
-            performs smart_fit
+        kind : {"std", "smart"}, default "std"
+            If "std", performs standard fit. If "smart",
+            performs a smart_fit - for more details see
+            the :ref:`User Guide <eels.fitting>`.
+        %s
 
-        **kwargs : key word arguments
-            Any extra key word argument will be passed to the chosen
-            fitter
+        Returns
+        -------
+        None
 
         See Also
         --------
-        multifit, smart_fit
+        * :py:meth:`~hyperspy.model.BaseModel.fit`
+        * :py:meth:`~hyperspy.model.BaseModel.multifit`
+        * :py:meth:`~hyperspy.model.EELSModel.smart_fit`
 
         """
-        if kind == 'smart':
-            self.smart_fit(fitter=fitter,
-                           method=method,
-                           grad=grad,
-                           bounded=bounded,
-                           ext_bounding=ext_bounding,
-                           update_plot=update_plot,
-                           **kwargs)
-        elif kind == 'std':
-            Model.fit(self,
-                      fitter=fitter,
-                      method=method,
-                      grad=grad,
-                      bounded=bounded,
-                      ext_bounding=ext_bounding,
-                      update_plot=update_plot,
-                      **kwargs)
-        else:
-            raise ValueError('kind must be either \'std\' or \'smart\'.'
-                             '\'%s\' provided.' % kind)
+        if kind not in ["smart", "std"]:
+            raise ValueError(
+                f"kind must be either 'std' or 'smart', not '{kind}'"
+            )
+        elif kind == "smart":
+            return self.smart_fit(**kwargs)
+        elif kind == "std":
+            return Model1D.fit(self, **kwargs)
+
+    fit.__doc__ %= FIT_PARAMETERS_ARG
 
     def smart_fit(self, start_energy=None, **kwargs):
-        """ Fits everything in a cascade style.
+        """Fits EELS edges in a cascade style.
+
+        The fitting procedure acts in iterative manner along
+        the energy-loss-axis. First it fits only the background
+        up to the first edge. It continues by deactivating all
+        edges except the first one, then performs the fit. Then
+        it only activates the the first two, fits, and repeats
+        this until all edges are fitted simultanously.
+
+        Other, non-EELSCLEdge components, are never deactivated,
+        and fitted on every iteration.
 
         Parameters
         ----------
-
         start_energy : {float, None}
             If float, limit the range of energies from the left to the
             given value.
-        **kwargs : key word arguments
-            Any extra key word argument will be passed to
-            the fit method. See the fit method documentation for
-            a list of valid arguments.
+        %s
 
         See Also
         --------
-        fit, multifit
+        * :py:meth:`~hyperspy.model.BaseModel.fit`
+        * :py:meth:`~hyperspy.model.BaseModel.multifit`
+        * :py:meth:`~hyperspy.model.EELSModel.fit`
 
         """
-
         # Fit background
         self.fit_background(start_energy, **kwargs)
 
         # Fit the edges
-        for i in xrange(0, len(self._active_edges)):
+        for i in range(0, len(self._active_edges)):
             self._fit_edge(i, start_energy, **kwargs)
 
+    smart_fit.__doc__ %= FIT_PARAMETERS_ARG
+
     def _get_first_ionization_edge_energy(self, start_energy=None):
         """Calculate the first ionization edge energy.
 
@@ -409,7 +408,7 @@ class EELSModel(Model):
         if iee is not None:
             to_disable = [edge for edge in self._active_edges
                           if edge.onset_energy.value >= iee]
-            E2 = iee - preferences.EELS.preedge_safe_window_width
+            E2 = iee - self._preedge_safe_window_width
             self.disable_edges(to_disable)
         else:
             E2 = None
@@ -437,11 +436,11 @@ class EELSModel(Model):
         """
         if powerlaw is None:
             for component in self._active_background_components:
-                if isinstance(component, components.PowerLaw):
+                if isinstance(component, components1d.PowerLaw):
                     if powerlaw is None:
                         powerlaw = component
                     else:
-                        messages.warning(
+                        _logger.warning(
                             'There are more than two power law '
                             'background components defined in this model, '
                             'please use the powerlaw keyword to specify one'
@@ -458,11 +457,11 @@ class EELSModel(Model):
                 E2 = ea[-1]
             else:
                 E2 = E2 - \
-                    preferences.EELS.preedge_safe_window_width
+                    self._preedge_safe_window_width
 
         if not powerlaw.estimate_parameters(
-                self.spectrum, E1, E2, only_current=False):
-            messages.warning(
+                self.signal, E1, E2, only_current=False):
+            _logger.warning(
                 "The power law background parameters could not "
                 "be estimated.\n"
                 "Try choosing a different energy range for the estimation")
@@ -473,61 +472,58 @@ class EELSModel(Model):
         ea = self.axis.axis[self.channel_switches]
         if start_energy is None:
             start_energy = ea[0]
-        preedge_safe_window_width = preferences.EELS.preedge_safe_window_width
         # Declare variables
-        edge = self._active_edges[edgenumber]
-        if edge.intensity.twin is not None or edge.active is False or \
-                edge.onset_energy.value < start_energy or edge.onset_energy.value > ea[-1]:
+        active_edges = self._active_edges
+        edge = active_edges[edgenumber]
+        if (edge.intensity.twin is not None or
+                edge.active is False or
+                edge.onset_energy.value < start_energy or
+                edge.onset_energy.value > ea[-1]):
             return 1
-        #~print "Fitting edge ", edge.name
-        last_index = len(self._active_edges) - 1
+        # Fitting edge 'edge.name'
+        last_index = len(self._active_edges) - 1    # Last edge index
         i = 1
         twins = []
-        #~print "Last edge index", last_index
+        # find twins
         while edgenumber + i <= last_index and (
-                self._active_edges[edgenumber + i].intensity.twin is not None or
-                self._active_edges[edgenumber + i].active is False):
-            if self._active_edges[edgenumber + i].intensity.twin is not None:
+                active_edges[edgenumber + i].intensity.twin is not None or
+                active_edges[edgenumber + i].active is False):
+            if active_edges[edgenumber + i].intensity.twin is not None:
                 twins.append(self._active_edges[edgenumber + i])
             i += 1
-        #~print "twins", twins
-        #~print "next_edge_index", edgenumber + i
         if (edgenumber + i) > last_index:
             nextedgeenergy = ea[-1]
         else:
-            nextedgeenergy = self._active_edges[edgenumber + i].onset_energy.value - \
-                preedge_safe_window_width
+            nextedgeenergy = (
+                active_edges[edgenumber + i].onset_energy.value -
+                self._preedge_safe_window_width)
 
         # Backup the fsstate
         to_activate_fs = []
         for edge_ in [edge, ] + twins:
-            if edge_.fine_structure_active is True and edge_.fine_structure_coeff.free is True:
+            if (edge_.fine_structure_active is True and
+                    edge_.fine_structure_coeff.free is True):
                 to_activate_fs.append(edge_)
         self.disable_fine_structure(to_activate_fs)
 
         # Smart Fitting
 
-        #~print("Fitting region: %s-%s" % (start_energy,nextedgeenergy))
-
         # Without fine structure to determine onset_energy
         edges_to_activate = []
         for edge_ in self._active_edges[edgenumber + 1:]:
-            if edge_.active is True and edge_.onset_energy.value >= nextedgeenergy:
+            if (edge_.active is True and
+                    edge_.onset_energy.value >= nextedgeenergy):
                 edge_.active = False
                 edges_to_activate.append(edge_)
-        #~print "edges_to_activate", edges_to_activate
-        #~print "Fine structure to fit", to_activate_fs
 
         self.set_signal_range(start_energy, nextedgeenergy)
         if edge.free_onset_energy is True:
-            #~print "Fit without fine structure, onset_energy free"
             edge.onset_energy.free = True
             self.fit(**kwargs)
             edge.onset_energy.free = False
-            print "onset_energy = ", edge.onset_energy.value
-            self._touch()
+            _logger.info("onset_energy = %s", edge.onset_energy.value)
+            self._classify_components()
         elif edge.intensity.free is True:
-            #~print "Fit without fine structure"
             self.enable_fine_structure(to_activate_fs)
             self.remove_fine_structure_data(to_activate_fs)
             self.disable_fine_structure(to_activate_fs)
@@ -536,7 +532,6 @@ class EELSModel(Model):
         if len(to_activate_fs) > 0:
             self.set_signal_range(start_energy, nextedgeenergy)
             self.enable_fine_structure(to_activate_fs)
-            #~print "Fit with fine structure"
             self.fit(**kwargs)
 
         self.enable_edges(edges_to_activate)
@@ -556,22 +551,23 @@ class EELSModel(Model):
                 if element not in elements:
                     elements[element] = {}
                 elements[element][subshell] = edge.intensity.value
-        print
-        print "Absolute quantification:"
-        print "Elem.\tIntensity"
+        print()
+        print("Absolute quantification:")
+        print("Elem.\tIntensity")
         for element in elements:
             if len(elements[element]) == 1:
                 for subshell in elements[element]:
-                    print "%s\t%f" % (
-                        element, elements[element][subshell])
+                    print("%s\t%f" % (
+                        element, elements[element][subshell]))
             else:
                 for subshell in elements[element]:
-                    print "%s_%s\t%f" % (element, subshell,
-                                         elements[element][subshell])
+                    print("%s_%s\t%f" % (element, subshell,
+                                         elements[element][subshell]))
 
     def remove_fine_structure_data(self, edges_list=None):
         """Remove the fine structure data from the fitting routine as
-        defined in the fine_structure_width parameter of the component.EELSCLEdge
+        defined in the fine_structure_width parameter of the
+        component.EELSCLEdge
 
         Parameters
         ----------
@@ -594,7 +590,8 @@ class EELSModel(Model):
         else:
             edges_list = [self._get_component(x) for x in edges_list]
         for edge in edges_list:
-            if edge.isbackground is False and edge.fine_structure_active is True:
+            if (edge.isbackground is False and
+                    edge.fine_structure_active is True):
                 start = edge.onset_energy.value
                 stop = start + edge.fine_structure_width
                 self.remove_signal_range(start, stop)
@@ -781,8 +778,8 @@ class EELSModel(Model):
         smart fit for the edges listed in edges_list.
         If edges_list is None (default) the onset_energy of all the edges
         with onset in the spectrum energy region will not be freed.
-        Note that if their atribute edge.onset_energy.free is True, the parameter
-        will be free during the smart fit.
+        Note that if their atribute edge.onset_energy.free is True, the
+        parameter will be free during the smart fit.
 
         Parameters
         ----------
@@ -867,8 +864,6 @@ class EELSModel(Model):
         for edge in edges_list:
             if edge.isbackground is False:
                 edge.intensity.free = True
-                #edge.onset_energy.free = True
-                #edge.fine_structure_coeff.free = True
 
     def fix_fine_structure(self, edges_list=None):
         """Fixes all the parameters of the edges given in edges_list.
@@ -932,8 +927,7 @@ class EELSModel(Model):
 
         See Also
         --------
-        resume_update
-        update_plot
+        resume_auto_fine_structure_width
 
         """
         if self._suspend_auto_fine_structure_width is False:
@@ -953,8 +947,7 @@ class EELSModel(Model):
 
         See Also
         --------
-        suspend_update
-        update_plot
+        suspend_auto_fine_structure_width
 
         """
         if self._suspend_auto_fine_structure_width is True:
